package keeper_test

import (
	"testing"

	sdk "github.com/cosmos/cosmos-sdk/types"
	sdkerrors "github.com/cosmos/cosmos-sdk/types/errors"
	"github.com/cosmos/cosmos-sdk/types/query"
	"github.com/stretchr/testify/require"
	"google.golang.org/grpc/codes"
	"google.golang.org/grpc/status"

	keepertest "github.com/cosmonaut/voter/testutil/keeper"
	"github.com/cosmonaut/voter/x/voter/types"
)

func TestL3NodeQuerySingle(t *testing.T) {
	keeper, ctx := keepertest.VoterKeeper(t)
	wctx := sdk.WrapSDKContext(ctx)
	msgs := createNL3Node(keeper, ctx, 2)
	for _, tc := range []struct {
		desc     string
		request  *types.QueryGetL3NodeRequest
		response *types.QueryGetL3NodeResponse
		err      error
	}{
		{
			desc:     "First",
			request:  &types.QueryGetL3NodeRequest{Id: msgs[0].Id},
			response: &types.QueryGetL3NodeResponse{L3Node: msgs[0]},
		},
		{
			desc:     "Second",
			request:  &types.QueryGetL3NodeRequest{Id: msgs[1].Id},
			response: &types.QueryGetL3NodeResponse{L3Node: msgs[1]},
		},
		{
			desc:    "KeyNotFound",
			request: &types.QueryGetL3NodeRequest{Id: uint64(len(msgs))},
			err:     sdkerrors.ErrKeyNotFound,
		},
		{
			desc: "InvalidRequest",
			err:  status.Error(codes.InvalidArgument, "invalid request"),
		},
	} {
		t.Run(tc.desc, func(t *testing.T) {
			response, err := keeper.L3Node(wctx, tc.request)
			if tc.err != nil {
				require.ErrorIs(t, err, tc.err)
			} else {
				require.NoError(t, err)
				require.Equal(t, tc.response, response)
			}
		})
	}
}

func TestL3NodeQueryPaginated(t *testing.T) {
	keeper, ctx := keepertest.VoterKeeper(t)
	wctx := sdk.WrapSDKContext(ctx)
	msgs := createNL3Node(keeper, ctx, 5)

	request := func(next []byte, offset, limit uint64, total bool) *types.QueryAllL3NodeRequest {
		return &types.QueryAllL3NodeRequest{
			Pagination: &query.PageRequest{
				Key:        next,
				Offset:     offset,
				Limit:      limit,
				CountTotal: total,
			},
		}
	}
	t.Run("ByOffset", func(t *testing.T) {
		step := 2
		for i := 0; i < len(msgs); i += step {
			resp, err := keeper.L3NodeAll(wctx, request(nil, uint64(i), uint64(step), false))
			require.NoError(t, err)
			require.LessOrEqual(t, len(resp.L3Node), step)
			require.Subset(t, msgs, resp.L3Node)
		}
	})
	t.Run("ByKey", func(t *testing.T) {
		step := 2
		var next []byte
		for i := 0; i < len(msgs); i += step {
			resp, err := keeper.L3NodeAll(wctx, request(next, 0, uint64(step), false))
			require.NoError(t, err)
			require.LessOrEqual(t, len(resp.L3Node), step)
			require.Subset(t, msgs, resp.L3Node)
			next = resp.Pagination.NextKey
		}
	})
	t.Run("Total", func(t *testing.T) {
		resp, err := keeper.L3NodeAll(wctx, request(nil, 0, 0, true))
		require.NoError(t, err)
		require.Equal(t, len(msgs), int(resp.Pagination.Total))
	})
	t.Run("InvalidRequest", func(t *testing.T) {
		_, err := keeper.L3NodeAll(wctx, nil)
		require.ErrorIs(t, err, status.Error(codes.InvalidArgument, "invalid request"))
	})
}
