package keeper

import (
	"encoding/binary"

	"github.com/cosmonaut/voter/x/voter/types"
	"github.com/cosmos/cosmos-sdk/store/prefix"
	sdk "github.com/cosmos/cosmos-sdk/types"
)

// GetL3NodeCount get the total number of l3Node
func (k Keeper) GetL3NodeCount(ctx sdk.Context) uint64 {
	store := prefix.NewStore(ctx.KVStore(k.storeKey), []byte{})
	byteKey := types.KeyPrefix(types.L3NodeCountKey)
	bz := store.Get(byteKey)

	// Count doesn't exist: no element
	if bz == nil {
		return 0
	}

	// Parse bytes
	return binary.BigEndian.Uint64(bz)
}

// SetL3NodeCount set the total number of l3Node
func (k Keeper) SetL3NodeCount(ctx sdk.Context, count uint64) {
	store := prefix.NewStore(ctx.KVStore(k.storeKey), []byte{})
	byteKey := types.KeyPrefix(types.L3NodeCountKey)
	bz := make([]byte, 8)
	binary.BigEndian.PutUint64(bz, count)
	store.Set(byteKey, bz)
}

// AppendL3Node appends a l3Node in the store with a new id and update the count
func (k Keeper) AppendL3Node(
	ctx sdk.Context,
	l3Node types.L3Node,
) uint64 {
	// Create the l3Node
	count := k.GetL3NodeCount(ctx)

	// Set the ID of the appended value
	l3Node.Id = count

	store := prefix.NewStore(ctx.KVStore(k.storeKey), types.KeyPrefix(types.L3NodeKey))
	appendedValue := k.cdc.MustMarshal(&l3Node)
	store.Set(GetL3NodeIDBytes(l3Node.Id), appendedValue)

	// Update l3Node count
	k.SetL3NodeCount(ctx, count+1)

	return count
}

// SetL3Node set a specific l3Node in the store
func (k Keeper) SetL3Node(ctx sdk.Context, l3Node types.L3Node) {
	store := prefix.NewStore(ctx.KVStore(k.storeKey), types.KeyPrefix(types.L3NodeKey))
	b := k.cdc.MustMarshal(&l3Node)
	store.Set(GetL3NodeIDBytes(l3Node.Id), b)
}

// GetL3Node returns a l3Node from its id
func (k Keeper) GetL3Node(ctx sdk.Context, id uint64) (val types.L3Node, found bool) {
	store := prefix.NewStore(ctx.KVStore(k.storeKey), types.KeyPrefix(types.L3NodeKey))
	b := store.Get(GetL3NodeIDBytes(id))
	if b == nil {
		return val, false
	}
	k.cdc.MustUnmarshal(b, &val)
	return val, true
}

// RemoveL3Node removes a l3Node from the store
func (k Keeper) RemoveL3Node(ctx sdk.Context, id uint64) {
	store := prefix.NewStore(ctx.KVStore(k.storeKey), types.KeyPrefix(types.L3NodeKey))
	store.Delete(GetL3NodeIDBytes(id))
}

// GetAllL3Node returns all l3Node
func (k Keeper) GetAllL3Node(ctx sdk.Context) (list []types.L3Node) {
	store := prefix.NewStore(ctx.KVStore(k.storeKey), types.KeyPrefix(types.L3NodeKey))
	iterator := sdk.KVStorePrefixIterator(store, []byte{})

	defer iterator.Close()

	for ; iterator.Valid(); iterator.Next() {
		var val types.L3Node
		k.cdc.MustUnmarshal(iterator.Value(), &val)
		list = append(list, val)
	}

	return
}

// GetL3NodeIDBytes returns the byte representation of the ID
func GetL3NodeIDBytes(id uint64) []byte {
	bz := make([]byte, 8)
	binary.BigEndian.PutUint64(bz, id)
	return bz
}

// GetL3NodeIDFromBytes returns ID in uint64 format from a byte array
func GetL3NodeIDFromBytes(bz []byte) uint64 {
	return binary.BigEndian.Uint64(bz)
}
