package keeper_test

import (
	"strconv"
	"testing"

	sdk "github.com/cosmos/cosmos-sdk/types"
	sdkerrors "github.com/cosmos/cosmos-sdk/types/errors"
	"github.com/stretchr/testify/require"

	keepertest "github.com/cosmonaut/voter/testutil/keeper"
	"github.com/cosmonaut/voter/x/voter/keeper"
	"github.com/cosmonaut/voter/x/voter/types"
)

// Prevent strconv unused error
var _ = strconv.IntSize

func TestAttachedUserMsgServerCreate(t *testing.T) {
	k, ctx := keepertest.VoterKeeper(t)
	srv := keeper.NewMsgServerImpl(*k)
	wctx := sdk.WrapSDKContext(ctx)
	creator := "A"
	for i := 0; i < 5; i++ {
		expected := &types.MsgCreateAttachedUser{Creator: creator,
			IidapId: int32(i),
			Account: int32(i),
		}
		_, err := srv.CreateAttachedUser(wctx, expected)
		require.NoError(t, err)
		rst, found := k.GetAttachedUser(ctx,
			expected.IidapId,
			expected.Account,
		)
		require.True(t, found)
		require.Equal(t, expected.Creator, rst.Creator)
	}
}

func TestAttachedUserMsgServerUpdate(t *testing.T) {
	creator := "A"

	for _, tc := range []struct {
		desc    string
		request *types.MsgUpdateAttachedUser
		err     error
	}{
		{
			desc: "Completed",
			request: &types.MsgUpdateAttachedUser{Creator: creator,
				IidapId: 0,
				Account: 0,
			},
		},
		{
			desc: "Unauthorized",
			request: &types.MsgUpdateAttachedUser{Creator: "B",
				IidapId: 0,
				Account: 0,
			},
			err: sdkerrors.ErrUnauthorized,
		},
		{
			desc: "KeyNotFound",
			request: &types.MsgUpdateAttachedUser{Creator: creator,
				IidapId: 100000,
				Account: 100000,
			},
			err: sdkerrors.ErrKeyNotFound,
		},
	} {
		t.Run(tc.desc, func(t *testing.T) {
			k, ctx := keepertest.VoterKeeper(t)
			srv := keeper.NewMsgServerImpl(*k)
			wctx := sdk.WrapSDKContext(ctx)
			expected := &types.MsgCreateAttachedUser{Creator: creator,
				IidapId: 0,
				Account: 0,
			}
			_, err := srv.CreateAttachedUser(wctx, expected)
			require.NoError(t, err)

			_, err = srv.UpdateAttachedUser(wctx, tc.request)
			if tc.err != nil {
				require.ErrorIs(t, err, tc.err)
			} else {
				require.NoError(t, err)
				rst, found := k.GetAttachedUser(ctx,
					expected.IidapId,
					expected.Account,
				)
				require.True(t, found)
				require.Equal(t, expected.Creator, rst.Creator)
			}
		})
	}
}

func TestAttachedUserMsgServerDelete(t *testing.T) {
	creator := "A"

	for _, tc := range []struct {
		desc    string
		request *types.MsgDeleteAttachedUser
		err     error
	}{
		{
			desc: "Completed",
			request: &types.MsgDeleteAttachedUser{Creator: creator,
				IidapId: 0,
				Account: 0,
			},
		},
		{
			desc: "Unauthorized",
			request: &types.MsgDeleteAttachedUser{Creator: "B",
				IidapId: 0,
				Account: 0,
			},
			err: sdkerrors.ErrUnauthorized,
		},
		{
			desc: "KeyNotFound",
			request: &types.MsgDeleteAttachedUser{Creator: creator,
				IidapId: 100000,
				Account: 100000,
			},
			err: sdkerrors.ErrKeyNotFound,
		},
	} {
		t.Run(tc.desc, func(t *testing.T) {
			k, ctx := keepertest.VoterKeeper(t)
			srv := keeper.NewMsgServerImpl(*k)
			wctx := sdk.WrapSDKContext(ctx)

			_, err := srv.CreateAttachedUser(wctx, &types.MsgCreateAttachedUser{Creator: creator,
				IidapId: 0,
				Account: 0,
			})
			require.NoError(t, err)
			_, err = srv.DeleteAttachedUser(wctx, tc.request)
			if tc.err != nil {
				require.ErrorIs(t, err, tc.err)
			} else {
				require.NoError(t, err)
				_, found := k.GetAttachedUser(ctx,
					tc.request.IidapId,
					tc.request.Account,
				)
				require.False(t, found)
			}
		})
	}
}
