package keeper

import (
	"encoding/binary"

	"github.com/cosmonaut/voter/x/voter/types"
	"github.com/cosmos/cosmos-sdk/store/prefix"
	sdk "github.com/cosmos/cosmos-sdk/types"
)

// GetRollupDigestCount get the total number of rollupDigest
func (k Keeper) GetRollupDigestCount(ctx sdk.Context) uint64 {
	store := prefix.NewStore(ctx.KVStore(k.storeKey), []byte{})
	byteKey := types.KeyPrefix(types.RollupDigestCountKey)
	bz := store.Get(byteKey)

	// Count doesn't exist: no element
	if bz == nil {
		return 0
	}

	// Parse bytes
	return binary.BigEndian.Uint64(bz)
}

// SetRollupDigestCount set the total number of rollupDigest
func (k Keeper) SetRollupDigestCount(ctx sdk.Context, count uint64) {
	store := prefix.NewStore(ctx.KVStore(k.storeKey), []byte{})
	byteKey := types.KeyPrefix(types.RollupDigestCountKey)
	bz := make([]byte, 8)
	binary.BigEndian.PutUint64(bz, count)
	store.Set(byteKey, bz)
}

// AppendRollupDigest appends a rollupDigest in the store with a new id and update the count
func (k Keeper) AppendRollupDigest(
	ctx sdk.Context,
	rollupDigest types.RollupDigest,
) uint64 {
	// Create the rollupDigest
	count := k.GetRollupDigestCount(ctx)

	// Set the ID of the appended value
	rollupDigest.Id = count

	store := prefix.NewStore(ctx.KVStore(k.storeKey), types.KeyPrefix(types.RollupDigestKey))
	appendedValue := k.cdc.MustMarshal(&rollupDigest)
	store.Set(GetRollupDigestIDBytes(rollupDigest.Id), appendedValue)

	// Update rollupDigest count
	k.SetRollupDigestCount(ctx, count+1)

	return count
}

// SetRollupDigest set a specific rollupDigest in the store
func (k Keeper) SetRollupDigest(ctx sdk.Context, rollupDigest types.RollupDigest) {
	store := prefix.NewStore(ctx.KVStore(k.storeKey), types.KeyPrefix(types.RollupDigestKey))
	b := k.cdc.MustMarshal(&rollupDigest)
	store.Set(GetRollupDigestIDBytes(rollupDigest.Id), b)
}

// GetRollupDigest returns a rollupDigest from its id
func (k Keeper) GetRollupDigest(ctx sdk.Context, id uint64) (val types.RollupDigest, found bool) {
	store := prefix.NewStore(ctx.KVStore(k.storeKey), types.KeyPrefix(types.RollupDigestKey))
	b := store.Get(GetRollupDigestIDBytes(id))
	if b == nil {
		return val, false
	}
	k.cdc.MustUnmarshal(b, &val)
	return val, true
}

// RemoveRollupDigest removes a rollupDigest from the store
func (k Keeper) RemoveRollupDigest(ctx sdk.Context, id uint64) {
	store := prefix.NewStore(ctx.KVStore(k.storeKey), types.KeyPrefix(types.RollupDigestKey))
	store.Delete(GetRollupDigestIDBytes(id))
}

// GetAllRollupDigest returns all rollupDigest
func (k Keeper) GetAllRollupDigest(ctx sdk.Context) (list []types.RollupDigest) {
	store := prefix.NewStore(ctx.KVStore(k.storeKey), types.KeyPrefix(types.RollupDigestKey))
	iterator := sdk.KVStorePrefixIterator(store, []byte{})

	defer iterator.Close()

	for ; iterator.Valid(); iterator.Next() {
		var val types.RollupDigest
		k.cdc.MustUnmarshal(iterator.Value(), &val)
		list = append(list, val)
	}

	return
}

// GetRollupDigestIDBytes returns the byte representation of the ID
func GetRollupDigestIDBytes(id uint64) []byte {
	bz := make([]byte, 8)
	binary.BigEndian.PutUint64(bz, id)
	return bz
}

// GetRollupDigestIDFromBytes returns ID in uint64 format from a byte array
func GetRollupDigestIDFromBytes(bz []byte) uint64 {
	return binary.BigEndian.Uint64(bz)
}
